from bge import logic
from bge import events

import math												

def Sign(value):
	if value > 0:
		return 1
	elif value < 0:
		return -1
	else:
		return 0
	
def Clamp(value, min, max):
	if value > max:
		return max
	elif value < min:
		return min
	else:
		return value
	
def AxisCheck(mx, my, size = 1.0, prop = 'wall'):

	cont = logic.getCurrentController()
	obj = cont.owner

	pos = obj.position									
	topos = pos.copy()									
	topos.x += (Sign(mx) * size) + mx					
	topos.y += (Sign(my) * size) + my					
	
	return obj.rayCast(topos, pos, 0, prop, 1, 1)		
	
def Player():

	cont = logic.getCurrentController()
	sce = logic.getCurrentScene()
	obj = cont.owner
	
	motion = cont.actuators['Motion']
	
	enemycol = cont.sensors['EnemyCol']
	
	key = logic.keyboard.events
	kbleft = key[events.AKEY]
	kbright = key[events.DKEY]
	kbup = key[events.WKEY]
	kbdown = key[events.SKEY]
	kbshoot = key[events.LKEY]

	def Init():
	
		if not 'init' in obj:
		
			obj['init'] = 1
			
			obj['mx'] = 0.0			
			obj['my'] = 0.0
			obj['accel'] = 0.05		
			obj['maxspd'] = 0.2		
			obj['friction'] = 0.7	
			
			obj['dir'] = 0.0		
			
			obj['atk'] = 5			
			obj['shottimer'] = 0.0	
			
			obj['player'] = 1		
	
	def Update():
	
		if enemycol.positive:
			print ("You've died. Great.")
			logic.restartGame()
	
		if kbleft > 0:
			obj['mx'] -= obj['accel']
		elif kbright > 0:
			obj['mx'] += obj['accel']
		else:
			obj['mx'] *= obj['friction']
		
		if kbup > 0:
			obj['my'] += obj['accel']
		elif kbdown > 0:
			obj['my'] -= obj['accel']
		else:
			obj['my'] *= obj['friction']
			
		if kbshoot:
		
			if obj['shottimer'] > .2:
			
				obj['shottimer'] = 0.0
				bullet = sce.addObject('Bullet', obj)
				bullet['spd'] = 1
				bullet['dir'] = obj['dir']
				bullet['type'] = 'Player'
				
		obj['shottimer'] += 1.0 / logic.getLogicTicRate()	
			
		obj['mx'] = Clamp(obj['mx'], -obj['maxspd'], obj['maxspd'])
		obj['my'] = Clamp(obj['my'], -obj['maxspd'], obj['maxspd'])
		
		if AxisCheck(obj['mx'], 0, .5)[0] != None:	
			obj['mx'] = 0	
			
		if AxisCheck(0, obj['my'], .5)[0] != None:	
			obj['my'] = 0  
		
		if AxisCheck(obj['mx'], obj['my'], .5)[0] != None:	
			obj['mx'] = 0									
			obj['my'] = 0
		
		motion.dLoc = [obj['mx'], obj['my'], 0.0]
		cont.activate(motion)
	
	def Animate():
		
		if max(abs(obj['my']), abs(obj['mx'])) > .1:	
		
			ori = obj.orientation.to_euler()		
			
			obj['dir'] = math.atan2(obj['my'], obj['mx'])	
			
			ori.z = obj['dir']	

			obj.orientation = ori
			
	Init()
	Update()
	Animate()
	
def Bullet():
	
	cont = logic.getCurrentController()
	obj = cont.owner
	sce = logic.getCurrentScene()
	cam = sce.active_camera
	
	def Init():
	
		if not 'init' in obj:
			obj['init'] = 1
			if not 'type' in obj:	
				obj['type'] = None	
			if not 'spd' in obj:
				obj['spd'] = 0.0
			if not 'dir' in obj:
				obj['dir'] = 0.0
	
	def Update():

		mx = math.cos(obj['dir']) * obj['spd']
		my = math.sin(obj['dir']) * obj['spd']
		
		if not cam.pointInsideFrustum(obj.position):
			obj.endObject()
		
		if AxisCheck(mx, my, .5)[0] != None:
			obj.endObject()
	
		obj.applyMovement([mx, my, 0.0], 0)			
	Init()
	Update()
	
	
